import json
import os

import whisper
from funutil import getLogger

from .base import BaseTask

logger = getLogger("funsketch")


class TextTask(BaseTask):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.success_file = os.path.join(self.sketch.result_text, "SUCCESS")

    def _run(self, *args, **kwargs):
        os.makedirs(self.sketch.result_text, exist_ok=True)
        files = os.listdir(self.sketch.result_audio)
        files = sorted(files, key=lambda x: x)
        model = whisper.load_model("turbo")
        for file in files:
            textfile = os.path.join(
                self.sketch.result_text, file.replace(".wav", ".txt")
            )
            if not file.endswith(".wav"):
                continue
            if os.path.exists(textfile):
                logger.info(f"text file {textfile} already exists")
                continue
            audio_path = os.path.join(self.sketch.result_audio, file)
            result = model.transcribe(audio_path, language="zh")
            with open(textfile, "w", encoding="utf-8") as f:
                f.write(json.dumps(result))



import os
from typing import List

from funutil import getLogger

from funsketch.sketch.meta import SketchMeta

logger = getLogger("funsketch")


class BaseTask:
    def __init__(self, sketch: SketchMeta, *args, **kwargs):
        self.sketch = sketch
        self.success_file = None

    def success(self):
        if self.success_file is not None:
            open(self.success_file, "a").close()

    def is_success(self):
        return self.success_file is not None and os.path.exists(self.success_file)

    def _run(self, *args, **kwargs):
        pass

    def run(self, retry=False, *args, **kwargs):
        if self.is_success():
            if not retry:
                logger.success(
                    f"task already success, {self.success_file} exists, skipping."
                )
                return
            os.remove(self.success_file)
        self._run(*args, **kwargs)
        self.success()


class TaskRun(BaseTask):
    def __init__(self, task_list: List[BaseTask], *args, **kwargs):
        self.task_list = task_list
        super().__init__(*args, **kwargs)

    def run(self, *args, **kwargs):
        for task in self.task_list:
            task.run(*args, **kwargs)



import os

from funutil import getLogger

logger = getLogger("funsketch")


class SketchMeta:
    def __init__(self, shared_url, pwd, name, root="./sketch_cache"):
        self.shared_url = shared_url
        self.pwd = pwd
        self.name = name
        self.root = f"{root}/{name}"
        self.result = os.path.join(self.root, "result")
        self.result_video = os.path.join(self.result, "video")
        self.result_audio = os.path.join(self.result, "audio")
        self.result_text = os.path.join(self.result, "text")
        logger.info(f"sketch name: {self.name}")
        logger.info(f"sketch root: {self.root}")



import os

from fundrive.drives.baidu.drive import BaiDuDrive
from funsecret import read_cache_secret
from funutil import getLogger

from .base import BaseTask

logger = getLogger("funsketch")


def longest_common_substring(strings):
    if not strings:
        return ""

    # 初始化第一个字符串
    base_string = strings[0]
    max_len = 0
    max_substr = ""

    # 遍历第一个字符串的所有子串
    for i in range(len(base_string)):
        for j in range(i + 1, len(base_string) + 1):
            substr = base_string[i:j]
            # 检查这个子串是否在所有字符串中
            if all(substr in s for s in strings):
                # 更新最长公共子串
                if len(substr) > max_len:
                    max_len = len(substr)
                    max_substr = substr

    return max_substr


class LoadTask(BaseTask):
    def __init__(self, bduss=None, stoken=None, ptoken=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        bduss = bduss or read_cache_secret("fundrive", "baidu", "bduss")
        stoken = stoken or read_cache_secret("fundrive", "baidu", "stoken")
        ptoken = ptoken or read_cache_secret("fundrive", "baidu", "ptoken")
        self.drive = BaiDuDrive()
        self.drive.login(bduss=bduss, stoken=stoken, ptoken=ptoken)
        self.success_file = os.path.join(self.sketch.result_video, "SUCCESS")

    def _run(self, *args, **kwargs):
        path = f"/sketch/{self.sketch.name}"
        if not self.drive.exist(path):
            self.drive.save_shared(
                shared_url=self.sketch.shared_url,
                remote_dir="/sketch",
                password=self.sketch.pwd,
            )
        os.makedirs(self.sketch.result_video, exist_ok=True)
        self.drive.download_dir(
            fid=path,
            local_dir=self.sketch.result_video,
            ignore_filter=lambda x: not x.endswith("mp4"),
            multi=True,
        )
        self.rename(self.sketch.result_video)

    def rename(self, path):
        files = [os.path.join(path, file) for file in os.listdir(path)]
        sizes = [len(file) for file in files]
        max_size = max(sizes)
        min_size = min(sizes)
        if min_size == max_size:
            logger.info("名称长度一样长")
            return
        common_name = longest_common_substring(files)
        logger.info("最长公共子串:", common_name)
        for old_name in files:
            new_name = (
                common_name
                + "0" * (max_size - len(old_name))
                + old_name.replace(common_name, "")
            )
            os.rename(old_name, new_name)



import os

from funutil import getLogger
from moviepy import VideoFileClip

from .base import BaseTask

logger = getLogger(__name__)


class AudioTask(BaseTask):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.success_file = os.path.join(self.sketch.result_audio, "SUCCESS")

    def _run(self, *args, **kwargs):
        os.makedirs(self.sketch.result_audio, exist_ok=True)
        files = os.listdir(self.sketch.result_video)
        files = sorted(files, key=lambda x: x)
        for file in files:
            filepath = os.path.join(self.sketch.result_video, file)
            if not filepath.endswith(".mp4"):
                continue
            video_path = filepath

            # 指定输出音频文件路径
            audio_path = os.path.join(
                self.sketch.result_audio, file.replace(".mp4", ".wav")
            )
            if os.path.exists(audio_path):
                logger.info(f"audio file {audio_path} already exists")
                continue
            video_clip = VideoFileClip(video_path)
            audio_clip = video_clip.audio
            audio_clip.write_audiofile(audio_path)
            audio_clip.close()
            video_clip.close()
